#!/usr/bin/env python3

# RAYCASTER
# Inspired by https://www.youtube.com/watch?v=gYRrGTC7GtA
#
# pip install pysdl2 pysdl2-dll

import sys
import sdl2.ext
import math

WIN_WIDTH = 640
WIN_HEIGHT = 480
DUNGEON_WIDTH = WIN_WIDTH
DUNGEON_HEIGHT = WIN_HEIGHT
PLAYER_SPEED = 10
RAY_LENGTH = 100

class Main:

	def __init__(self):
		# Graphics
		sdl2.ext.init()
		self.window = sdl2.ext.Window("Raycaster", size=(WIN_WIDTH, WIN_HEIGHT))
		self.window.show()
		self.surface = self.window.get_surface()

		# Player
		self.player_position = {"x": int(DUNGEON_WIDTH/2), "y": int(DUNGEON_HEIGHT/2), "r": 0}	# r is rotation in radiants

		return

	def run(self):
		running = True
		while running:
			events = sdl2.ext.get_events()
			for event in events:
				if event.type == sdl2.SDL_QUIT:
					running = False
					break
				if event.type == sdl2.SDL_KEYDOWN:
					# Rotate player
					if event.key.keysym.sym == sdl2.SDLK_LEFT:
						self.player_position["r"] = self.player_position["r"] - 0.1
					elif event.key.keysym.sym == sdl2.SDLK_RIGHT:
						self.player_position["r"] = self.player_position["r"] + 0.1

					# Compute deltax and deltay based on player direction
					player_delta_x = math.cos(self.player_position["r"]) * PLAYER_SPEED
					player_delta_y = math.sin(self.player_position["r"]) * PLAYER_SPEED

					# Move player based on its direction
					if event.key.keysym.sym == sdl2.SDLK_UP:
						self.player_position["y"] = int(self.player_position["y"] + player_delta_y)
						self.player_position["x"] = int(self.player_position["x"] + player_delta_x)
					elif event.key.keysym.sym == sdl2.SDLK_DOWN:
						self.player_position["y"] = int(self.player_position["y"] - player_delta_y)
						self.player_position["x"] = int(self.player_position["x"] - player_delta_x)

					# Limit position into dungeon bounds
					if self.player_position["x"] < 0:
						self.player_position["x"] = 0
					if self.player_position["x"] > DUNGEON_WIDTH:
						self.player_position["x"] = DUNGEON_WIDTH
					if self.player_position["y"] < 0:
						self.player_position["y"] = 0
					if self.player_position["y"] > DUNGEON_HEIGHT:
						self.player_position["y"] = DUNGEON_HEIGHT
					if self.player_position["r"] > 2*math.pi:
						self.player_position["r"] = 0
					if self.player_position["r"] < 0:
						self.player_position["r"] = 2*math.pi
					print(self.player_position)

			sdl2.ext.draw.fill(self.surface, sdl2.ext.Color(0,0,0,0)) # Clears screen
			self.draw()
			self.window.refresh()
		return 0

	def draw(self):
		# Player in 2D map
		sdl2.ext.draw.fill(self.surface, sdl2.ext.Color(0,255,0,255), (self.player_position["x"] - 2, self.player_position["y"] - 2, 4, 4))
		# Player line of sight in 2D map
		ray = {
			"x": int(self.player_position["x"] + math.cos(self.player_position["r"]) * 50), # deltaX + playerX
			"y": int(self.player_position["y"] + math.sin(self.player_position["r"]) * 50)  # deltaY + playerY
		}
		print(self.player_position)
		sdl2.ext.draw.line(self.surface, sdl2.ext.Color(255,0,0,255), (self.player_position["x"], self.player_position["y"], ray["x"], ray["y"]))
		return



if __name__ == '__main__':
	try:
		main = Main()
		main.run()
	except KeyboardInterrupt:
		exit(0)
